###################################################################################################
# _metacmdmanager.py
#
# Implementation of class MetacmdManager (see below).
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from Globals import HTML, HTMLFile
from Products.PythonScripts import PythonScript
import string
import urllib
import copy
# Product Imports.
import _globals
import _xmllib


# Example code.
# -------------

dtmlMethodExampleCode = \
  '<dtml-comment>\n' + \
  '# Example code:\n' + \
  '</dtml-comment>\n' + \
  '\n' + \
  '<dtml-call expr="REQUEST.set(\'message\',\'This is %s\'%meta_type)">\n' + \
  '<dtml-return message>\n' + \
  ''

pyScriptExampleCode = \
  '# Example code:\n' + \
  '\n' + \
  '# Import a standard function, and get the HTML request and response objects.\n' + \
  'from Products.PythonScripts.standard import html_quote\n' + \
  'request = container.REQUEST\n' + \
  'RESPONSE =  request.RESPONSE\n' + \
  '\n' + \
  '# Return a string identifying this script.\n' + \
  'print "This is the", script.meta_type, \'"%s"\' % script.getId(),\n' + \
  'if script.title:\n' + \
  '    print "(%s)" % html_quote(script.title),\n' + \
  'print "in", container.absolute_url()\n' + \
  'return printed\n' + \
  ''


# -------------------------------------------------------------------------------------------------
#  importMetaCmd
# -------------------------------------------------------------------------------------------------
def importMetaCmd(self, xml):
  builder = _xmllib.XmlAttrBuilder()
  value = builder.parse(xml)
  id = value['id']
  # Catalog.
  meta_cmds = self.getMetaCmds()
  meta_cmd = {}
  meta_cmd['id'] = id
  meta_cmd['name'] = value['name']
  meta_cmd['description'] = value['description']
  meta_cmd['meta_types'] = value['meta_types']
  meta_cmd['roles'] = value['roles']
  meta_cmd['exec'] = value.has_key('exec') and value['exec']
  meta_cmds.append(meta_cmd)
  self.__attr_dc_metacmds__ = meta_cmds
  # Object.
  if value['meta_type'] == 'DTML Method':
    title = '*** DO NOT DELETE OR MODIFY ***'
    self.manage_addDTMLMethod(id,title)
    ob = getattr(self,id)
    ob.manage_edit(title=ob.title,data=value['data'])
  elif value['meta_type'] == 'Script (Python)':
    PythonScript.manage_addPythonScript(self,id)
    ob = getattr(self,id)
    ob.ZPythonScript_edit(params='',body=value['data'])
  return id


###################################################################################################
###################################################################################################
###
###   class MetacmdObject
###
###################################################################################################
###################################################################################################
class MetacmdObject: 

    ############################################################################################
    # MetacmdManager.manage_executeMetacmd:
    #
    # Execute Meta-Command.
    ############################################################################################
    def manage_executeMetacmd(self, custom, lang, manage_lang, REQUEST, RESPONSE):
        """ MetacmdManager.manage_executeMetacmd """
        
        message = ''
        
        for meta_cmd in self.getMetaCmds():
          if meta_cmd['name'] == custom:
            ob = getattr(self,meta_cmd['id'],None)
            if ob is not None:
              if ob.meta_type == 'DTML Method':
                message = ob(self,REQUEST,RESPONSE)
              elif ob.meta_type == 'Script (Python)':
                message = ob()

        # Return with message.
        message = urllib.quote(message)
        return RESPONSE.redirect('manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(lang,manage_lang,message))
        


###################################################################################################
###################################################################################################
###
###   class MetacmdManager
###
###################################################################################################
###################################################################################################
class MetacmdManager: 

    # Management Permissions.
    # -----------------------
    __authorPermissions__ = ()
    __editorPermissions__ = ()
    __administratorPermissions__ = (
		'manage_changeMetacmds','manage_customizeMetacmdForm',
		)
    __ac_permissions__=(
		('ZMS Administrator', __administratorPermissions__),
		('ZMS Author', __authorPermissions__),
		('ZMS Editor', __editorPermissions__),
		)


    # Management Interface to customize Meta-Functions.
    manage_customizeMetacmdForm = HTMLFile('dtml/metacmd/manage_customizeform', globals()) 


    # ------------------------------------------------------------------------------------------
    #  getMetaCmds
    # ------------------------------------------------------------------------------------------
    def getMetaCmds(self):
      if not hasattr(self,'__attr_dc_metacmds__'):
        self.__attr_dc_metacmds__ = []
      metacmds = getattr(self,'__attr_dc_metacmds__')
      for metacmd in metacmds:
        if not metacmd.has_key('exec'):
          metacmd['exec'] = 1
      return metacmds
    
        
    ############################################################################################
    # MetacmdManager.manage_changeMetacmds:
    #
    # Change Meta-Commands.
    ############################################################################################
    def manage_changeMetacmds(self, btn, lang, manage_lang, REQUEST, RESPONSE):
        """ MetacmdManager.manage_changeMetacmds """

        message = ''
        id = ''
    
        # Insert.
        # -------
        if btn == self.getLangStr('BTN_INSERT',manage_lang):
          id = REQUEST['_id']
          # Catalog.
          meta_cmds = self.getMetaCmds()
          meta_cmd = {}
          meta_cmd['id'] = id
          meta_cmd['name'] = REQUEST.get('_name','')
          meta_cmd['description'] = ''
          meta_cmd['meta_types'] = []
          meta_cmd['roles'] = []
          meta_cmd['exec'] = REQUEST.get('_exec',0)
          meta_cmds.append(meta_cmd)
          self.__attr_dc_metacmds__ = meta_cmds
          # Object.
          if REQUEST.get('_type','DTML Method') == 'DTML Method':
            title = '*** DO NOT DELETE OR MODIFY ***'
            self.manage_addDTMLMethod(id,title)
            ob = getattr(self,id)
            ob.manage_edit(title=ob.title,data=dtmlMethodExampleCode)
          elif REQUEST.get('_type','DTML Method') == 'Script (Python)':
            PythonScript.manage_addPythonScript(self,id)
            ob = getattr(self,id)
            ob.ZPythonScript_edit(params='',body=pyScriptExampleCode)
            
        # Delete.
        # -------
        elif btn == self.getLangStr('BTN_DELETE',manage_lang):
          id = REQUEST['id']
          # Catalog.
          meta_cmds = self.getMetaCmds()
          for meta_cmd in meta_cmds:
            if meta_cmd['id'] == id:
              meta_cmds.remove(meta_cmd)
          self.__attr_dc_metacmds__ = meta_cmds
          # Object.
          self.manage_delObjects(ids=[id])

        # Change.
        # -------
        elif btn == self.getLangStr('BTN_CHANGE',manage_lang):
          id = REQUEST['el_id']
          old_id = REQUEST['id']
          # Catalog.
          meta_cmds = self.getMetaCmds()
          for meta_cmd in meta_cmds:
            if meta_cmd['id'] == old_id:
              meta_cmd['id'] = id
              meta_cmd['name'] = REQUEST.get('el_name','')
              meta_cmd['description'] = REQUEST.get('el_description','')
              meta_cmd['meta_types'] = REQUEST.get('el_meta_types',[])
              meta_cmd['roles'] = REQUEST.get('el_roles',[])
              meta_cmd['exec'] = REQUEST.get('el_exec',0)
              meta_cmd = meta_cmd.copy()
          self.__attr_dc_metacmds__ = meta_cmds
          # Object.
          if id != old_id:
            self.manage_renameObject(id=old_id,new_id=id)
          ob = getattr(self,id)
          if ob.meta_type == 'DTML Method':
            ob.manage_edit(title=ob.title,data=REQUEST.get('el_data',''))
          elif ob.meta_type == 'Script (Python)':
            ob.ZPythonScript_edit(params='',body=REQUEST.get('el_data',''))

        # Import.
        # -------
        elif btn == self.getLangStr('BTN_IMPORT',manage_lang):
          id = importMetaCmd(self,REQUEST['file'])

        # Export.
        # -------
        elif btn == self.getLangStr('BTN_EXPORT',manage_lang):
          id = REQUEST['id']
          # Catalog.
          name = ''
          description = ''
          meta_types = []
          roles = []
          meta_cmds = self.getMetaCmds()
          for meta_cmd in meta_cmds:
            if meta_cmd['id'] == id:
              el_name = meta_cmd['name']
              el_description = meta_cmd['description']
              el_meta_types = meta_cmd['meta_types']
              el_roles = meta_cmd['roles']
              el_exec = meta_cmd['exec']
          # Object.
          ob = getattr(self,id)
          el_meta_type = ob.meta_type
          if ob.meta_type == 'DTML Method':
            el_data = ob.raw
          else:
            el_data = ob.body()
          # XML.
          content_type = 'text/xml'
          filename = id + '.xml'
          export = '<?xml version="1.0" encoding="ISO-8859-1"?>'
          export = export + _xmllib.toXml(self,{'id':id,'name':el_name,'description':el_description,'meta_types':el_meta_types,'roles':el_roles,'exec':el_exec,'meta_type':el_meta_type,'data':el_data})
          RESPONSE.setHeader('Content-Type',content_type)
          RESPONSE.setHeader('Content-Disposition','inline;filename=%s'%filename)
          return export

        # Return with message.
        message = urllib.quote(message)
        return RESPONSE.redirect('manage_customizeMetacmdForm?lang=%s&manage_lang=%s&manage_tabs_message=%s&id=%s'%(lang,manage_lang,message,id))

###################################################################################################
