###################################################################################################
# _metadata.py
#
# Implementation of class Metadata (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from Globals import HTMLFile
import string
# Product Imports.
import _globals


###################################################################################################
###################################################################################################
###
###   C L A S S   M E T A D A T A
###
###################################################################################################
###################################################################################################
class Metadata:

    # Management Interface.
    metadata_header = HTMLFile('dtml/metadict/header', globals()) 
    metadata_input_fields = HTMLFile('dtml/metadict/input_fields', globals()) 
    metadata_input_field = HTMLFile('dtml/metadict/input_field', globals()) 

    # ------------------------------------------------------------------------------------------
    #	Metadata.getMetaAttrs:
    # 
    #	Returns list of Object-Attributes of DC.Metadata.
    # ------------------------------------------------------------------------------------------
    def getMetaAttrs(self, REQUEST, meta_type=None):
      meta_type = _globals.nvl(meta_type,self.meta_type)
      meta_attrs = []
      if len(self.getLanguages()) > 1:
        meta_attrs.extend(['attr_dc_coverage'])
      meta_attrs.extend(self.getMetadictAttrs(meta_type))
      return meta_attrs


    # ---------------------------------------------------------------------------------------------
    #  Metadata.getDCCoverageOptpl:
    #
    #  Values for select-list of DC.Coverage.
    #
    #  @returns opttpl: Tuple (value, display) of options.
    # ---------------------------------------------------------------------------------------------
    def getDCCoverageOptpl(self, REQUEST, meta_type=None):
      meta_type = _globals.nvl(meta_type,self.meta_type)
      
      keys = []
      if meta_type == 'ZMSLinkElement':
        keys.append(REQUEST['lang'])
      else:
        coverage = self.getDCCoverage(REQUEST)
        if REQUEST.get('ZMS_INSERT',None) is None:
          if REQUEST['lang'] == self.getPrimaryLanguage():
            if coverage == 'obligation':
              keys.extend(['obligation'])
            elif coverage == 'proposal':
              keys.extend(['obligation','proposal'])
            elif coverage == REQUEST['lang']:
              keys.extend(['obligation','proposal',REQUEST['lang']])
          else:
            keys.extend([coverage])
        else:
          if REQUEST['lang'] == self.getPrimaryLanguage():
            if coverage == 'obligation':
              keys.extend(['obligation','proposal'])
            elif coverage == 'proposal':
              keys.extend(['proposal'])
          keys.extend([REQUEST['lang']])
      
      optpl = []
      for key in keys:
        if key in self.getLanguages():
          optpl.append([key,"local (%s)"%self.getLanguageLabel(key)])
        else:
          optpl.append([key,key])
      
      return optpl


    # ---------------------------------------------------------------------------------------------
    #	Metadata.acquireMetadata:
    #
    #	Acquires Metadata from parent.
    # ---------------------------------------------------------------------------------------------
    def acquireMetadata(self, key, lang, manage_lang, REQUEST):
      parent = self.getParentNode()
      if parent is not None and key in parent.getObjAttrs().keys():
        # Set property.          
        self.setObjStateModified(REQUEST)
        self.setObjProperty(key,parent.getObjProperty(key,REQUEST),lang)
        self.onChangeObj(REQUEST)


    # ---------------------------------------------------------------------------------------------
    #	Metadata.inheritMetadata:
    #
    #	Inherits Metadata recursive to children.
    # ---------------------------------------------------------------------------------------------
    def inheritMetadata(self, key, lang, manage_lang, REQUEST):
      for child in self.getChildNodes(REQUEST):
        if key in child.getObjAttrs().keys():
          # Set property.          
          child.setObjStateModified(REQUEST)
          child.setObjProperty(key,self.getObjProperty(key,REQUEST),lang)
          child.onChangeObj(REQUEST)
          # Process tree.          
          child.inheritMetadata(key,lang,manage_lang,REQUEST)


    # ---------------------------------------------------------------------------------------------
    #	Metadata.setMetadata:
    #
    #	Changes Metadata.
    # ---------------------------------------------------------------------------------------------
    def setMetadata(self, lang, manage_lang, REQUEST):
    
      # Acquire from top.
      if REQUEST.get('aq_dir','') == 'top':
        if REQUEST.has_key('aq_ids'):
          for key in REQUEST['aq_ids']:
            self.acquireMetadata(key,lang,manage_lang,REQUEST)
                
      # Inherit to bottom.
      elif REQUEST.get('aq_dir','') == 'bottom':
        if REQUEST.has_key('aq_ids'):
          for key in REQUEST['aq_ids']:
            self.inheritMetadata(key,lang,manage_lang,REQUEST)
                
      # Change.
      elif REQUEST.get('btn','') != self.getLangStr('BTN_CANCEL',manage_lang):
        for attr in self.getMetaAttrs(REQUEST):
          if attr in self.getMetadictAttrs(self.meta_type):
            metadictAttr = self.getMetadictAttr(attr)
          else:
            metadictAttr = {'key':attr,'type':'string'}
          if metadictAttr['type'] in self.getMetaobjIds(sort=0):
            for ob in self.getObjChildren(metadictAttr['id'],REQUEST):
              ##### Object State ####
              ob.setObjStateModified(REQUEST)
              ##### Properties ####
              for key in self.getMetaobjAttrIds(metadictAttr['type']):
                ob.setReqProperty(key,REQUEST)
              ##### VersionManager ####
              ob.onChangeObj(REQUEST)
          else:
            self.setReqProperty(metadictAttr['key'],REQUEST)

###################################################################################################