###################################################################################################
# _objattrs.py
#
# Implementation of class ObjAttrs (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
import DocumentTemplate.DT_HTML
import copy
import string
import sys
import time
import urllib
# Product Imports.
import _blobfields
import _globals


# ---------------------------------------------------------------------------------------------
#  _objattrs.hasobjattr:
# ---------------------------------------------------------------------------------------------
def hasobjattr(obj, key):
  try:
    return key in obj.__dict__.keys()
  except:
    return hasattr(obj,key)

# ---------------------------------------------------------------------------------------------
#  _objattrs.getobjattr:
# ---------------------------------------------------------------------------------------------
def getobjattr(self, obj, obj_attr, lang):
  key = self.getObjAttrName(obj_attr,lang)
  if hasobjattr(obj,key):
    return getattr(obj,key)
  else:
    if obj_attr.has_key('default'):
      return obj_attr['default']
    datatype = obj_attr['datatype_key']
    if datatype == _globals.DT_BOOLEAN:
      return 0
    elif datatype == _globals.DT_DICTIONARY:
      return {}
    elif datatype == _globals.DT_LIST:
      return []
    elif datatype in _globals.DT_DATETIMES:
      return None 
    elif datatype in _globals.DT_NUMBERS:
      return 0
    elif datatype in _globals.DT_STRINGS:
      return ''
    elif datatype in _globals.DT_BLOBS:
      return None 
    return None

# ---------------------------------------------------------------------------------------------
#  _objattrs.setobjattr:
# ---------------------------------------------------------------------------------------------
def setobjattr(self, obj, obj_attr, value, lang):
  key = self.getObjAttrName(obj_attr,lang)
  # Assign value.
  setattr(obj,key,value)

# ---------------------------------------------------------------------------------------------
#  _objattrs.cloneobjattr:
# ---------------------------------------------------------------------------------------------
def cloneobjattr(self, src, dst, obj_attr, lang):
  # Fetch value.
  value = getobjattr(self,src,obj_attr,lang)
  # Clone value.
  if value is not None:
    datatype = obj_attr['datatype_key']
    if datatype in _globals.DT_BLOBS:
      value = value._getCopy()
    else:
      try:
        value = copy.deepcopy(value)
      except:
        try:
          value = value.copy()
        except:
          value = value
  # Assign value.
  setobjattr(self,dst,obj_attr,value,lang)



###################################################################################################
###################################################################################################
###
###   O B J E C T   A T T R I B U T E S
###
###################################################################################################
###################################################################################################
class ObjAttrs:

    # ---------------------------------------------------------------------------------------------
    #	ObjAttrs.getObjAttrs:
    #
    #	Returns object-attributes and resolves meta-dictionaries.
    # ---------------------------------------------------------------------------------------------
    def getObjAttrs(self, meta_type=None):
      if self.meta_type == 'ZMSAttributeContainer':
        return self.aq_parent.getObjAttrs(meta_type)
      if meta_type is None:
        if self.meta_type == 'ZMSCustom':
          meta_type = self.meta_id
        else:
          meta_type = self.meta_type
      return self.dObjAttrs.get(meta_type,{})

    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.getObjAttr:
    # ---------------------------------------------------------------------------------------------
    def getObjAttr(self, key, meta_type=None):
      obj_attrs = self.getObjAttrs(meta_type)
      ##### Attribute ####      
      if obj_attrs.has_key(key):
        obj_attr = obj_attrs[key]
      ##### Default ####        
      else:
        obj_attr = {}
        obj_attr['key'] = key
        obj_attr['xml'] = 1
        obj_attr['datatype'] = None
        obj_attr['default'] = ''
        obj_attr['multilang'] = 0
        obj_attr['lang_inherit'] = _globals.LI_ALLOWED
        obj_attr['datatype_key'] = _globals.DT_UNKNOWN
      # Return Object-Attribute.
      return obj_attr


    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.getObjOptions:
    # ---------------------------------------------------------------------------------------------
    def getObjOptions(self, obj_attr, REQUEST):
      optpl = []
      if obj_attr.has_key('options'):
        opts = []
        obj_attropts = obj_attr['options']
        if type(obj_attropts) is type([]):
          for i in range(0,len(obj_attropts)/2):
            opts.append([obj_attropts[i*2],obj_attropts[i*2+1]])
        elif type(obj_attropts) is type({}):
          for k in obj_attropts.keys():
            opts.append([k,obj_attropts[k]])
        for opt in opts:
          lang_attr = 'OPT_'
          for skey in string.split(obj_attr['key'],'_'):
            lang_attr = lang_attr + skey[0]
          lang_attr = lang_attr + '_' + string.replace(str(opt[1]),' ','')
          lang_attr = string.upper(lang_attr)
          lang_display = self.getLangStr(lang_attr,REQUEST['manage_lang'])
          value = str(opt[0])
          display = value
          if lang_attr != lang_display:
            display = lang_display
          optpl.append([value,display])
      return optpl

    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.getObjAttrName:
    # ---------------------------------------------------------------------------------------------
    def getObjAttrName(self, obj_attr, lang=None):
      attr = obj_attr['key']
      if obj_attr['multilang']:
        attr = '%s_%s'%(attr,lang)
      return attr

    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.isDisabledAttr:
    # ---------------------------------------------------------------------------------------------
    def isDisabledAttr(self, obj_attr, REQUEST):
      prim_lang = self.getPrimaryLanguage()
      lang = REQUEST.get('lang',prim_lang)
      multilang = obj_attr['multilang']
      coverage = self.getDCCoverage(REQUEST)
      enabled = 0
      enabled = enabled or (self.getAutocommit())
      enabled = enabled or (prim_lang == lang)
      enabled = enabled or (not multilang and coverage == lang)
      enabled = enabled or (multilang and coverage != 'obligation')
      enabled = enabled or (multilang and obj_attr['lang_inherit'] == _globals.LI_ALLOWED)
      return not enabled


    ###############################################################################################
    #
    #  I N T E R F A C E
    #
    ###############################################################################################

    # ---------------------------------------------------------------------------------------------
    #  getObjInput:
    # ---------------------------------------------------------------------------------------------
    def getObjInput(self, key, REQUEST):

      #-- DEFINITION
      meta_type = REQUEST.get('ZMS_INSERT',None)
      obj_attr = self.getObjAttr(key,meta_type)
      
      #-- DATATYPE
      datatype = obj_attr['datatype_key']
      
      #-- NAME
      lang = REQUEST['lang']
      manage_lang = REQUEST['manage_lang']
      fmName = 'form0'
      elName = self.getObjAttrName(obj_attr,lang)
      
      #-- INPUTTYPE
      inputtype = obj_attr.get('type','string')
      
      #-- VALUE
      if meta_type is None:
        value = self.getObjAttrValue(obj_attr,REQUEST)
      elif key == 'active':
        value = 1
      else:
        value = ''
      
      #-- ENABLED
      enabled = not self.isDisabledAttr(obj_attr,REQUEST)
      
      #-- Image-Fields.
      if inputtype == 'image':
        return self.f_selectImage(self,ob=self,fmName=fmName,elName=elName,key=key,lang=lang,manage_lang=manage_lang,REQUEST=REQUEST)
      
      #-- File-Fields.
      elif inputtype == 'file':
        return self.f_selectFile(self,ob=self,fmName=fmName,elName=elName,key=key,lang=lang,manage_lang=manage_lang,REQUEST=REQUEST)
      
      #-- Password-Fields.
      if inputtype == 'password':
        size = obj_attr.get('size',20)
        return self.getPasswordInput(fmName,elName,size,value)
      
      #-- Dictionary-Fields.
      elif inputtype == 'dictionary':
        css = 'form-fixed'
        wrap = 'off'
        cols = 40
        rows = 15
        return self.getTextArea(fmName,elName,cols,rows,self.toXmlString(value),enabled,REQUEST,css,wrap)
      
      #-- Text-Fields.
      elif inputtype == 'text':
        css = 'form-element'
        wrap = 'on'
        if hasattr(self,'getFormat'):
          s_textformat = self.getFormat(REQUEST)
          o_textformat = self.getTextFormat(s_textformat,REQUEST)
          if not o_textformat is None and not o_textformat.getTag() and not o_textformat.getSubTag():
            css = 'form-fixed'
            wrap = 'off'
        cols = obj_attr.get('size',40)
        if cols <= 40:
          rows = 5
        else:
          rows = 15
        return self.getTextArea(fmName,elName,cols,rows,value,enabled,REQUEST,css,wrap)
      
      #-- Boolean-Fields.
      elif inputtype == 'boolean':
        return self.getCheckbox(fmName,elName,value,1,enabled,0,REQUEST)
      
      #-- Select-Fields.
      elif inputtype in ['select','multiselect']:
        lang_str = self.getLangStr(obj_attr.get('label',string.upper(key)),manage_lang)
        required = obj_attr.get('mandatory',0)
        optpl = self.getObjOptions(obj_attr,REQUEST)
        return self.getSelect(fmName,elName,value,inputtype,lang_str,required,optpl,enabled,REQUEST)
      
      #-- Input-Fields.
      else: 
        if datatype in _globals.DT_DATETIMES:
          size = 12
          fmt_str = 'DATETIME_FMT'
          if datatype == _globals.DT_DATE:
            size = 8
            fmt_str = 'DATE_FMT'
          elif datatype == _globals.DT_TIME:
            size = 8
            fmt_str = 'TIME_FMT'
          return self.getDateTimeInput(fmName,elName,size,value,enabled,fmt_str,REQUEST)
        elif datatype == _globals.DT_URL:
          elTextName = obj_attr.get('textattr','')
          if len(elTextName) > 0:
            elTextName = self.getObjAttrName(self.getObjAttr(elTextName,meta_type),REQUEST['lang'])
          size = 25
          return self.getUrlInput(fmName,elName,elTextName,size,value,enabled,REQUEST)
        else:
          size = 20
          if obj_attr.has_key('size'):
            size = obj_attr['size']
          elif datatype == _globals.DT_INT:
            size = 5
          elif datatype == _globals.DT_FLOAT:
            size = 8
          return self.getTextInput(fmName,elName,size,value)


    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.blob_html:
    # ---------------------------------------------------------------------------------------------
    def blob_html(self, key, REQUEST, RESPONSE):
      """ ObjAttrs.blob_html """
      return self.getObjProperty(key,REQUEST).blob_html(REQUEST,RESPONSE)


    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.hasObjProperty:
    #
    #  Checks if object has specified property.
    # ---------------------------------------------------------------------------------------------
    def hasObjProperty(self, key, REQUEST):

      #-- REQUEST
      lang = REQUEST.get('lang',self.getPrimaryLanguage())
      
      #-- DEFINITION
      obj_attr = self.getObjAttr(key)
      
      #-- OBJECT (Live / Work).
      ob = self.getObjVersion(REQUEST)
      
      #-- ATTR
      attr = self.getObjAttrName(obj_attr,lang)
      
      #-- Return if object has property (0=false, 1=true).
      return hasobjattr(ob,attr)


    """
    ###############################################################################################
    #
    #  G E T
    #
    ###############################################################################################
    """

    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs._getObjAttrValue:
    # ---------------------------------------------------------------------------------------------
    def _getObjAttrValue(self, obj_attr, datatype, REQUEST):

      #-- REQUEST
      lang = REQUEST.get('lang',self.getPrimaryLanguage())
      
      #-- OBJECT (Live / Work).
      ob = self.getObjVersion(REQUEST)
      
      #-- VALUE
      set, value = 0, getobjattr(self,ob,obj_attr,lang)
      
      #-- None List-Fields
      if datatype not in [_globals.DT_UNKNOWN,_globals.DT_LIST] and type(value) is type([]):
        if len(value) > 0:
          set, value = 1, value[0]
        else:
          value = None
      
      #-- Blob-Fields
      if datatype in _globals.DT_BLOBS:
        if type(value) is type(''):
          set, value = 1, None
        elif value is not None:
          value = value._getCopy()
          value.aq_parent = self
          value.key = obj_attr['key']
          value.lang = lang
      
      #-- List-Fields.
      elif datatype == _globals.DT_LIST and type(value) is type(''):
        set, value = 1, [value]
      
      #-- Integer-Fields.
      elif datatype in _globals.DT_INTS and type(value) is type('') and len(value) > 0:
        try:
          set, value = 1, int(value)
        except:
          set, value = 1, obj_attr.get('default',0)
      
      #-- Float-Fields.
      elif datatype == _globals.DT_FLOAT and type(value) is type('') and len(value) > 0:
        try:
          set, value = 1, float(value)
        except:
          set, value = 1, obj_attr.get('default',0.0)
      
      #-- DateTime-Fields.
      elif datatype in _globals.DT_DATETIMES:
        if value is not None and type(value) is type(''): 
          fmt_str = 'DATETIME_FMT'
          if datatype == _globals.DT_DATE:
            fmt_str = 'DATE_FMT'
          elif datatype == _globals.DT_TIME:
            fmt_str = 'TIME_FMT'
          set, value = 1, self.parseLangFmtDate(value,lang,fmt_str)
        value = _globals.getDateTime(value)
      
      #-- SET?
      if set: 
        attr = self.getObjAttrName(obj_attr,lang)
        _globals.writeLog("[%s._getObjAttrValue]: setattr(%s,%s)"%(self.meta_type,attr,value))
        setattr(ob,attr,value)
      
      # Return value.
      return value

    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.getObjAttrValue:
    # ---------------------------------------------------------------------------------------------
    def getObjAttrValue(self, obj_attr, REQUEST):
    
      #-- REQUEST
      lang = REQUEST.get('lang',self.getPrimaryLanguage())
      
      #-- DATATYPE
      datatype = obj_attr['datatype_key']

      #-- VALUE
      value = self._getObjAttrValue(obj_attr,datatype,REQUEST)

      # Default to parent-language.
      parent_lang = self.getParentLanguage(lang)
      if parent_lang is not None and \
         parent_lang != lang and \
         obj_attr['multilang'] and \
         obj_attr['lang_inherit'] != _globals.LI_FORBIDDEN:
        empty = 0
        empty = empty or (value is None)
        empty = empty or (datatype in _globals.DT_NUMBERS and obj_attr.get('default',0) > 0 and value == 0)
        empty = empty or (datatype in _globals.DT_STRINGS and len(str(value)) == 0)
        if empty or self.isDisabledAttr(obj_attr,REQUEST):
          req = {'lang':parent_lang,'preview':'preview'}
          value = self.getObjAttrValue(obj_attr,req)

      # Return value.
      return value

    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.getObjProperty:
    #
    #  Retrieves value for specified property.
    # ---------------------------------------------------------------------------------------------
    def getObjProperty(self, key, REQUEST={}, default=None):

      #-- [ReqBuff]: Fetch buffered value from Http-Request.
      try:
        reqBuffId = '%s_%s'%('getObjProperty',key)
        value = self.fetchReqBuff(reqBuffId,REQUEST)
        return value
      except:
      
        #-- Get value.
        obj_attr = self.getObjAttr(key)
        value = self.getObjAttrValue(obj_attr,REQUEST)
        
        #-- Process dtml.
        datatype = obj_attr['datatype_key']
        if datatype in _globals.DT_STRINGS:
          if value is not None and type(value) is type(''):
            if not self.getConfProperty('Demo.Active') == 1:
              try:
                dt_html = DocumentTemplate.DT_HTML.HTML(value)
                value = dt_html(self,REQUEST)
              except:
                value = '[Exception]:<br>'
                value = value + 'Type: <b>%s</b><br>'%str(sys.exc_type)
                value = value + 'Value: <b>%s</b><br>'%str(sys.exc_value)
                value = value + 'Traceback: <b>%s</b><br>'%str(sys.exc_traceback)
        #-- [ReqBuff]: Returns value and stores it in buffer of Http-Request.
        return self.storeReqBuff(reqBuffId,value,REQUEST)


    """
    ###############################################################################################
    #
    #  S E T
    #
    ###############################################################################################
    """
    
    # ---------------------------------------------------------------------------------------------
    #  setReqProperty:
    #
    #  Assigns value to specified property from Request-Object.
    # ---------------------------------------------------------------------------------------------
    def setReqProperty(self, key, REQUEST):
    
      #-- REQUEST
      lang = REQUEST['lang']
      manage_lang = REQUEST['manage_lang']
      
      #-- DEFINTION
      obj_attr = self.getObjAttr(key)
      
      #-- DATATYPE
      datatype = obj_attr['datatype_key']
      
      #-- RETURN
      if not obj_attr['xml'] or datatype == _globals.DT_UNKNOWN: return
      
      #-- ATTR
      attr = self.getObjAttrName(obj_attr,lang)
      
      #-- VALUE
      set, value = 0, REQUEST.get(attr,None)
      
      #-- Blob-Fields
      if datatype in _globals.DT_BLOBS:
      
        # Image-Field (Dimensions)
        if datatype == _globals.DT_IMAGE:
          img = self._getObjAttrValue(obj_attr,datatype,REQUEST)
          if img is not None:
            if REQUEST.has_key('width_%s'%attr): 
              img.width = REQUEST['width_%s'%attr]
            if REQUEST.has_key('height_%s'%attr): 
              img.height = REQUEST['height_%s'%attr]
            self.setObjProperty(key,img,lang)
        
        # Delete
        if REQUEST.has_key('del_%s'%attr) and int(REQUEST['del_%s'%attr]) == 1:
          set, value = 1, None
        
        # Upload
        elif value is not None and len(getattr(value,'filename','')) > 0:
          set, value = 1, value
        
        # Reset
        elif len(REQUEST.get('ZMS_INSERT','')) > 0:
          set, value = 1, None
      
      #-- DateTime-Fields.
      elif datatype in _globals.DT_DATETIMES:
        if value is not None and type(value) is type(''):
          fmt_str = 'DATETIME_FMT'
          if datatype == _globals.DT_DATE:
            fmt_str = 'DATE_FMT'
          elif datatype == _globals.DT_TIME:
            fmt_str = 'TIME_FMT'
          set, value = 1, self.parseLangFmtDate(value,lang,fmt_str)
        value = _globals.getDateTime(value)
      
      #-- Dictionary-Fields
      elif datatype == _globals.DT_DICTIONARY:
        if value is not None:
          if type(value) is type(''):
            value = self.parseXmlString('<?xml version="1.0"?>'+value)
          if type(value) is type({}):
            set, value = 1, value
      
      #-- List-Fields
      elif datatype == _globals.DT_LIST:
        if value is None:
          value = []
        if type(value) is type(''):
          value = [string.strip(value)]
        if type(value) is type([]):
          set, value = 1, value
      
      #-- String-Fields
      elif datatype in _globals.DT_STRINGS:
        if value is not None:
          set, value = 1, value
      
      #-- Integer-Fields
      elif datatype in _globals.DT_INTS:
        if value is not None:
          if type(value) == type('') and len(value) == 0:
            set, value = 1, None
          else:
            set, value = 1, int(value)
        else:
          set, value = 1, 0
      
      #-- Float-Fields
      elif datatype == _globals.DT_FLOAT:
        if value is not None:
          if type(value) == type('') and len(value) == 0:
            set, value = 1, None
          else:
            set, value = 1, float(value)
        else:
          set, value = 1, 0.0
      
      #-- SET?
      if set: 
        _globals.writeLog("[%s.setReqProperty] %s=%s"%(self.meta_type,key,str(value)))
        self.setObjProperty(key,value,lang)

    
    # ---------------------------------------------------------------------------------------------
    #  setObjProperty:
    #
    #  Assigns value to specified property.
    # ---------------------------------------------------------------------------------------------
    def setObjProperty(self, key, value, lang=None, forced=0):
    
        #-- DEFINITION
        obj_attr = self.getObjAttr(key)
        
        #-- DATATYPE
        datatype = obj_attr['datatype_key']
        
        #-- VALUE
        if type(value) is type(''):
          value = string.strip(value)
          while len(value) > 0 and value[-1] == '\n':
            value = value[:-1]
        # Retrieve value from options.
        if obj_attr.has_key('options'):
          options = obj_attr['options']
          try: 
            i = options.index(int(value))
            if i%2==1: value = options[i-1]
          except:
            try:
              i = options.index(str(value))
              if i%2==1: value = options[i-1]
            except:
              pass
        
        #-- Blob-Fields
        if datatype in _globals.DT_BLOBS:
          file = value
          if file is not None and \
             len(str(file)) > 0 and \
             not (str(file) == str(_blobfields.createBlobField(self,datatype,'',lang))):
            value = _blobfields.createBlobField(self,datatype,file,lang)
            
        #-- DateTime-Fields.
        elif datatype in _globals.DT_DATETIMES:
          if value is not None and type(value) is type(''):
            fmt_str = 'DATETIME_FMT'
            if datatype == _globals.DT_DATE:
              fmt_str = 'DATE_FMT'
            elif datatype == _globals.DT_TIME:
              fmt_str = 'TIME_FMT'
            value = self.parseLangFmtDate(value,lang,fmt_str)
          value = _globals.getDateTime(value)
        
        #-- Dictionary-Fields
        elif datatype == _globals.DT_DICTIONARY:
          value = value.copy()
        
        #-- Integer-Fields
        elif datatype in _globals.DT_INTS:
          if value is not None and type(value) is type('') and len(value) > 0:
            value = int(value)
        
        #-- Float-Fields
        elif datatype == _globals.DT_FLOAT:
          if value is not None and type(value) is type('') and len(value) > 0:
            value = float(value)
        
        #-- String-Fields.
        elif datatype in _globals.DT_STRINGS:
          if value is None:
            value = ''
          else:
            if type(value) is type(''):
              value = string.strip(value)
        
        #-- Url-Fields
        if datatype == _globals.DT_URL:
          #++ if _globals.debug: print "[%s.setObjProperty]: %s(URL)"%(self.meta_type,key)
          req = {'lang':lang,'prevew':'preview'}
          old_value = self.getObjProperty(key,req)
          # Unregister old reference.
          old_ref_obj = self.getLinkObj(old_value)
          if old_ref_obj is not None:
            old_ref_obj.unregisterRefObj(self,req)
          # Register new reference.
          ref_obj = self.getLinkObj(value)
          if ref_obj is not None:
            ref_obj.registerRefObj(self,req)
            # Repair link (if necessary).
            value = self.getRefObjPath(ref_obj)
        
        #-- SET!
        #++ _globals.writeLog("[%s.setObjProperty]: %s(%s)=%s"%(self.meta_type,key,str(datatype),str(value)))
        ob = self.getObjVersion({'preview':'preview'})
        setobjattr(self,ob,obj_attr,value,lang)
        if forced:
          ob = self.getObjVersion()
          setobjattr(self,ob,obj_attr,value,lang)


    """
    ###############################################################################################
    #
    #  C L O N E
    #
    ###############################################################################################
    """
    
    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.cloneObjAttrs:
    #
    #  Clone object-attributes.
    # ---------------------------------------------------------------------------------------------
    def cloneObjAttrs(self, src, dst, REQUEST={}):
    
      #-- REQUEST
      lang = REQUEST.get('lang',self.getPrimaryLanguage())
      
      #-- Clone
      for key in self.getObjAttrs().keys():
        obj_attr = self.getObjAttr(key)
        # Multi-Language Attributes.
        if obj_attr['multilang']:
          for s_lang in self.getLanguages():
            if lang in ['*',s_lang]:
              cloneobjattr(self,src,dst,obj_attr,s_lang)
        # Others.
        else:
          if lang in ['*',self.getPrimaryLanguage(),self.getDCCoverage(REQUEST)]:
            cloneobjattr(self,src,dst,obj_attr,lang)


    """
    ###############################################################################################
    #
    #  D a t a   ( I m a g e / F i l e )
    #
    ###############################################################################################
    """

    #----------------------------------------------------------------------------------------------
    #	ObjAttrs.recurse_downloadRessources:
    #
    #	Download from ZODB to file-system during Export.
    #----------------------------------------------------------------------------------------------
    def recurse_downloadRessources(self, folder='.'):
      obj_attrs = self.getObjAttrs()
      _globals.writeLog("[%s.recurse_downloadRessources]: %s"%(self.meta_type,self.id))
      message = ''
      # Attributes.
      for key in obj_attrs.keys():
        obj_attr = self.getObjAttr(key)
        datatype = obj_attr['datatype_key']
        if datatype in _globals.DT_BLOBS:
          for s_lang in self.getLanguages():
            try:
              req = {}
              req['lang'] = s_lang
              req['preview'] = 'preview'
              ob = self._getObjAttrValue(obj_attr,datatype,req)
              if ob is not None: 
                filepath = self.absolute_url()[len(self.getDocumentElement().absolute_url()):]
                ob.aq_parent = self
                ob.key = key
                ob.download('%s/%s'%(folder,filepath))
            except:
              _globals.writeLog("*****[%s.recurse_downloadRessources]: Exception in %s.%s_%s!*****"%(self.meta_type,self.id,key,s_lang))
              _globals.writeLog("-  Type: %s"%str(sys.exc_type))
              _globals.writeLog("-  Value: %s"%str(sys.exc_value))
              _globals.writeLog("-  Traceback: %s"%str(sys.exc_traceback))
              pass
      # Process children.
      for child in self.getChildNodes():
        message = message + child.recurse_downloadRessources(folder)
      # Return message.
      return message


    #----------------------------------------------------------------------------------------------
    #	ObjAttrs.recurse_uploadRessources:
    #
    #	Upload from file-system to ZODB during Import.
    #----------------------------------------------------------------------------------------------
    def recurse_uploadRessources(self, folder='.'):
      obj_attrs = self.getObjAttrs()
      _globals.writeLog("[%s.recurse_uploadRessources]: %s"%(self.meta_type,self.id))
      message = ''
      # Attributes.
      for key in obj_attrs.keys():
        obj_attr = self.getObjAttr(key)
        datatype = obj_attr['datatype_key']
        if datatype in _globals.DT_BLOBS:
          for s_lang in self.getLanguages():
            try:
              req = {}
              req['lang'] = s_lang
              req['preview'] = 'preview'
              ob = self._getObjAttrValue(obj_attr,datatype,req)
              if ob is not None:
                ob.aq_parent = self
                ob.key = key
                ob.lang = s_lang
                ob.upload(folder)
                self.setObjProperty(key,ob,s_lang)
                self.onChangeObj(req,forced=1)
            except:
              _globals.writeLog("*****[%s.recurse_uploadRessources]: Exception in %s.%s_%s!*****"%(self.meta_type,self.id,key,s_lang))
              _globals.writeLog("-  Type: %s"%str(sys.exc_type))
              _globals.writeLog("-  Value: %s"%str(sys.exc_value))
              _globals.writeLog("-  Traceback: %s"%str(sys.exc_traceback))
              pass
      # Process children.
      for child in self.getChildNodes():
        message = message + child.recurse_uploadRessources(folder)
      # Return message.
      return message


###################################################################################################
###################################################################################################
###
###   O B J E C T   A T T R I B U T E S   M A N A G E R
###
###################################################################################################
###################################################################################################
class ObjAttrsManager:

    # ---------------------------------------------------------------------------------------------
    #	ObjAttrsManager.synchronizeObjAttrs:
    #
    #	Synchronizes object-attributes.
    # ---------------------------------------------------------------------------------------------
    def synchronizeObjAttrs(self):
      #++ if _globals.debug: print '[%s.synchronizeObjAttrs]'%(self.meta_type)

      datatypes = ['boolean','date','datetime','dictionary','file','float','image','int','multiselect','password','select','string','text','time','url']
      self.dObjAttrs = {}
      for meta_type in self.dGlobalAttrs.keys():
        
        ##### Special Objects ####
        if meta_type == 'ZMSCustom':
          for meta_id in self.getMetaobjIds(sort=0):
            obj_attrs = self.dGlobalAttrs[meta_type]['obj_attrs'].copy()
            for key in self.getMetaobjAttrIds(meta_id):
              attr = self.getMetaobjAttr(meta_id,key)
              if attr['type'] in datatypes:
                dct = {}
                dct['xml'] = 1
                dct['key'] = attr['id']
                dct['label'] = attr['name']
                dct['type'] = attr['type']
                dct['datatype'] = attr['type']
                if attr['type'] == 'password':
                  dct['type'] = attr['type']
                  dct['datatype'] = 'string'
                elif attr['type'] == 'select':
                  dct['type'] = attr['type']
                  dct['datatype'] = 'string'
                elif attr['type'] == 'multiselect':
                  dct['type'] = attr['type']
                  dct['datatype'] = 'list'
                dct['mandatory'] = attr['mandatory']
                dct['multilang'] = attr['multilang']
                if len(attr['keys'])>0:
                  options = []
                  for option in attr['keys']:
                    options.append(option)
                    options.append(option)
                  dct['options'] = options
                obj_attrs[dct['key']] = dct
            self.dObjAttrs[meta_id] = obj_attrs
      
        ##### Standard Objects (Meta-Dictionaries) ####
        else:
          obj_attrs = self.dGlobalAttrs[meta_type]['obj_attrs'].copy()
          if obj_attrs.has_key('$metadict'):
            del obj_attrs['$metadict']
            for key in self.getMetadictAttrs(meta_type):
              attr = self.getMetadictAttr(key)
              if attr['type'] in datatypes:
                dct = {}
                dct['key'] = attr['key']
                dct['label'] = attr['name']
                dct['type'] = attr['type']
                dct['datatype'] = attr['type']
                if attr['type'] == 'password':
                  dct['type'] = attr['type']
                  dct['datatype'] = 'string'
                elif attr['type'] == 'select':
                  dct['type'] = attr['type']
                  dct['datatype'] = 'string'
                elif attr['type'] == 'multiselect':
                  dct['type'] = attr['type']
                  dct['datatype'] = 'list'
                dct['mandatory'] = attr['mandatory']
                dct['multilang'] = attr['multilang']
                if len(attr['keys'])>0:
                  options = []
                  for option in attr['keys']:
                    options.append(option)
                    options.append(option)
                  dct['options'] = options
                obj_attrs[dct['key']] = dct
          self.dObjAttrs[meta_type] = obj_attrs

      ##### Defaults ####
      for meta_type in self.dObjAttrs.keys():
        obj_attrs = self.dObjAttrs[meta_type]
        for key in obj_attrs.keys():
          obj_attr = obj_attrs[key]
          obj_attr['key'] = key
          obj_attr['xml'] = obj_attr.get('xml',1)
          obj_attr['datatype'] = obj_attr.get('datatype','string')
          obj_attr['mandatory'] = obj_attr.get('mandatory',0)
          obj_attr['multilang'] = obj_attr.get('multilang',0)
          obj_attr['lang_inherit'] = obj_attr.get('lang_inherit',_globals.LI_ALLOWED)
          #-- DATATYPE_KEY
          datatype = string.lower(obj_attr['datatype'])
          obj_attr['datatype_key'] = _globals.datatype_key(datatype)
          obj_attr['type'] = obj_attr.get('type',datatype)
          # Apply defaults.
          obj_attrs[key] = obj_attr
        self.dObjAttrs[meta_type] = obj_attrs

###################################################################################################